	SUBROUTINE LIST_ERROR(CLI_REQUEST_BLOCK,ERROR_CODE)
	IMPLICIT INTEGER (A-Z)
C++
C
C FUNCTION:
C
C	THIS MODULE IS CALLED BY DCL PARSE ROUTINES AS A RESULT
C	OF A SYNTAX ERROR IN THE COMMAND LINE
C
C INPUTS:
C
C	CLI_REQUEST_BLOCK	CLI REQUEST BLOCK CONTAINING DESCRIPTOR
C				OF ERROR
C	ERROR_CODE		STANDARD VAX CODE DESCRIBING ERROR
C
C OUTPUTS:
C
C	INFORMATION DESCRIBING THE ERROR IS EXTRACTED FROM THE
C	REQUEST BLOCK AND THE ERROR CONDITION IS SIGNALED.
C
C SIDE EFFECTS:
C
C	THE DONE BIT IS SET FORCING AN ABORT.
C
C--

	LOGICAL*1 CLI_REQUEST_BLOCK(0:0)

	INTEGER*4 ERROR_CODE
	INTEGER*4 ERROR_SEGMENT_DESCR(2)	! DESCRIBES BAD SEGMENT
	INTEGER*4 UNPROCESSED_TEXT_DESCR(2)	! DESCRIBES INPROCESSED COMMAND
	INTEGER*4 COMMAND_STARTING_ADDRESS	! ADDRESS OF 1ST BYTE OF COMMAND
	INTEGER*4 COMMAND_ENDING_ADDRESS	! ADDRESS OF LAST BYTE OF COMMAND
	INTEGER*4 COMMAND_LENGTH		! LENGTH OF COMMAND (BYTES)
	INTEGER*4 START_OF_UNPROCESSED_TEXT	! ADDRESS OF NEXT BYTE IN COMMAND
	INTEGER*4 UNPROCESSED_TEXT_LENGTH	! BYTES REMAINING IN COMMAND

	INCLUDE 'CLI.INC'
	INCLUDE 'STATUS.INC'

	PARAMETER LENGTH 		= 1	! OFFSET TO DESCRIPTOR LENGTH FIELD
	PARAMETER ADDRESS		= 2	! OFFSET TO DESCRIPTOR ADDRESS FIELD

C
C	SAY WE ARE DONE SO MAIN LINE WILL ABORT
C

	LIST_STATUS = LIST_STATUS .OR. LIST_M_DONE

C
C	COPY ERROR SEGMENT DESCRIPTOR
C

	CALL COPY(8,
	1         CLI_REQUEST_BLOCK(CLI$Q_RQDESC),
	2         ERROR_SEGMENT_DESCR(LENGTH))

C
C	FIND STARTING ADDRESS OF COMMAND
C

	CALL COPY(4,
	1         %VAL(REQUEST(PARSE_COMMAND)+CLI$A_RQADDR),
	2         COMMAND_STARTING_ADDRESS)

C
C	NOW GET LENGTH OF COMMAND
C

	CALL COPY(2,
	1         %VAL(REQUEST(PARSE_COMMAND)+CLI$W_RQSIZE),
	2         COMMAND_LENGTH)

C
C	PRINT UP TO 80 BYTES OF UNPROCESSED COMMAND TEXT.
C

	COMMAND_ENDING_ADDRESS = COMMAND_STARTING_ADDRESS + COMMAND_LENGTH - 1

	START_OF_UNPROCESSED_TEXT = ERROR_SEGMENT_DESCR(ADDRESS) +
	1                           ERROR_SEGMENT_DESCR(LENGTH)

	UNPROCESSED_TEXT_LENGTH = MIN0(80,COMMAND_ENDING_ADDRESS -
	1                                 START_OF_UNPROCESSED_TEXT + 1)

	UNPROCESSED_TEXT_DESCR(LENGTH) = UNPROCESSED_TEXT_LENGTH

	UNPROCESSED_TEXT_DESCR(ADDRESS) = START_OF_UNPROCESSED_TEXT

C
C	NOW SIGNAL ERROR CONDITON
C

	CALL LIB$SIGNAL(%VAL(%LOC(ERROR_CODE)),
	1               %VAL(0),			! NO FAO ARG FOR 1ST MESSAGE
	2               %VAL(CLI$_BADTEXT),		! 2ND ERROR CODE
	3               %VAL(2),			! 2 FAO ARGUMENTS
	4               ERROR_SEGMENT_DESCR,
	5               UNPROCESSED_TEXT_DESCR)

	RETURN
	END


