#include "[levy.vaxc]eric.h"

/*

	       Regular expression compiler and matcher.
		(originally from David Conroys Grep on rsx)

     recmp(regexp,string)    char regexp[],string[];

        Takes the regular expression in regexp compiles it into PBUF
        and  then  compares the input string against it. If a match,
        then 1 is returned else 0. 

     compile(regexp)         char regexp[];

        Separately compiles the input regular  expression  into  the
        global character string `PBUF' for later use in match. 

     match(string)           char string[];

        Compares  the  input  string against PBUF and returns 1 if a
        match  otherwise  it  returns  0.  On  a  match  the  global
        variables  __loc1,__loc2 contain the address in string where
        the match occured and the address  of  then  next  character
        beyond where the match ended. Thus, __loc2 - __loc1 computes
        the length of the match. 


Recognized regular expression patterns are:

	c       Matches the character itself (c is any non special
		character). Letters are matched caselessly.

        [c..c]  character class (matches any of the characters inside
		the brackets.

        [^c..c] Negated character character class (matches any character
		not inside the brackets.

	.       Matches any single character.

	@       Matches any letter a-z or A-Z  (setable ranges - see below)

	#       Matches any number 0-9         (setable ranges)

	*       Closure, Matches 0 or more occurances of the preceding
		regular expression.

	^       Anchors the string to the beginning, i.e. the remaining
		regular expression must match at the beginning of the
		string.

	$       Anchors the string to the end, i.e. the preceding regular
		expression must match upto the end of the line.

	\c      Matches c where c is any character at all, i.e. turns
		off the special meanings of characters.

		white space (a blank or ascii space ) is equivalent to
		space or tabs.

    The variables __num1, __num2 and __alf1,__alf2  contain  `0',  `9'
    and  `a',  `z'  for  ranges  of  numbers  and letters. The # and @
    metacharacters use these for the range of characters they compare.
    Note that alpha folds  all  characters  to  lower  case  before  a
    compare.  Thus # can be utilized for any range of ascii characters
    while @ is more restrictive. The user  must  change  these  ranges
    before any calls to regular expression routines, and they stay set
    forever after. 


*/

#define	E_ND	0
#define CHAR	1
#define BOL	2
#define EOL	3
#define	ANY	4
#define CLASS	5
#define	NCLASS	6
#define CLOSURE	7
#define ALPHA	8
#define NUMBER	9
#define WHITE   10
#define PMAX 128

/*

        The global variables holding the character pointer, and  the
        pattern buffer. 

*/

char    *pp = {0};
char pbuf[PMAX] = {0};

static store(),lower(),badpat();
static char *cclass(),*pmatch();
char *__loc1 = 0,*__loc2 = 0;
int   __num1 = '0', __num2 = '9';
int   __alf1 = 'a', __alf2 = 'z';


#define mprogx
#ifdef mprog

function main(argc,argv) int argc;  char *argv[]; {
	int x;
	x = recmp(argv[1],argv[2]);
       eprintf("match re=(%s):str=(%s) = %d\n",argv[1],argv[2],x);
	}

#endif

function         recmp(reg,str) char reg[],str[]; {
	compile(reg);
	return (match(str));
	}

function         compile(s)
register char *s;
    {
    register char *lp;
    register int c;
    int o;
    char *spp;
    
    pp = pbuf;
    while(c = *s++)
        {
        
        /*
        * Closure is special.
        */
        if(c == '*')
            {
            
            if(pp==pbuf || (o=pp[-1])==BOL || o==EOL || o==CLOSURE)
            badpat();
            store(E_ND);
            store(E_ND);
            spp = pp;
            while(--pp > lp)
            *pp = pp[-1];
            *pp = CLOSURE;
            pp = spp;
            continue;
            
            }
        
        /*
        * All the rest.
        */
        lp = pp;
        switch(c)
            {
            
            
            case '^':
            store(BOL);
            break;
            
            case '$':
            store(EOL);
            break;
            
	    case ' ':
	    store(WHITE);
            break;
            
            case '.':
            store(ANY);
            break;
            
            case '[':
            s = cclass(s);
            break;
            
            case '\\':
		if (*s) then
                    c = *s++;
                    store(CHAR);
                    store(lower(c));
		fi
                break;
            
            case '@':
            store(ALPHA);
            break;
                        
            case '#':
            store(NUMBER);
            break;

            default:
            store(CHAR);
            store(lower(c));
            
            }
        
        }
    store(E_ND);
    }

function static  char *cclass(s)
register char *s;
    {
    register char *cp;
    register int c;
    int o;
    
    o = CLASS;
    if(*s == '^')
        {
        
        ++s;
        o = NCLASS;
        }
    store(o);
    cp = pp;
    store(0);  /* Byte count */
    while((c = *s++) && c!=']')
        {
        
        if(c == '\\')
        if((c = *s++) == '\0')
        badpat();
        store(lower(c));
        if(++*cp == 0)
        error("Class too complex\n");
        }
    if(c != ']')
    badpat();
    return(s);
    }

function static  store(op)
    {
    if(pp >= &pbuf[PMAX])
    error("Pattern too complex\n");
    *pp++ = op;
    }

function         match(lbuf) char lbuf[];
    {
    register char *lll; register char *mmm
    
    lll = lbuf;
    while(*lll)
        {
        
	if (mmm = pmatch(lll, pbuf,lbuf)) then
/*	   printe("matches with lll=%d lbuf=%d mmm=%d",lll,lbuf,mmm); */
	   __loc1 = lll; __loc2 = mmm;
           return(1);
	els
           ++lll;
	fi
        }
    return(0);
    }

function static  char *pmatch(l, p,lbuf)  register char *l, *p; char lbuf[];
    {
    register char *e;
    int op, c, n;
    char *are;
    
    while((op = *p++) != E_ND)
    switch(op)
        {
        
        case ALPHA:
        c=lower(*l++);
	if(c<__alf1 | c>__alf2)
        return (0);
        break;
                
        case NUMBER:
        c = *l++;
	if(c<__num1 | c>__num2)
        return (0);
        break;
        
        case CHAR:
        if(lower(*l++) != *p++)
        return(0);
        break;
        
        case BOL:
        if(l != lbuf)
        return(0);
        break;
        
        case EOL:
        if(*l != '\0')
        return(0);
        break;
        
	case WHITE:
	c = lower(*l++);
	if(c != ' ' && c != '\t')
            return(0);
        break;
        
        case ANY:
        if(*l++ == '\0')
        return(0);
        break;
        
        case CLASS:
        c = lower(*l++);
        if((n = *p++&0377) == 0)
        return(0);
        do
            {
            
            if(c == *p++)
            break;
            
            }
        while(--n);
        if(n == 0)
        return(0);
        p += n-1;
        break;
        
        case NCLASS:
        c = lower(*l++);
        if((n = *p++&0377) == 0)
        break;
        do
            {
            
            if(c == *p++)
            break;
            
            }
        while(--n);
        if(n)
        return(0);
        break;
        
        case CLOSURE:
        are = l;
        while(e = pmatch(l, p,lbuf))
        l = e;
        while(*p++ != E_ND)
        ;
        while(l >= are)
            {
            
            if(e = pmatch(l, p,lbuf))
            return(e);
            --l;
            
            }
        
        return(0);
        
        default:
        error("Cannot happen -- match\n");
        
        }
    
    return(l);
    }

function static  lower(c)
register int c;
    {
    if(c>='A' && c<='Z')
    c += 'a'-'A';
    return(c);
    }

function static  badpat()
    {
    error("Bad pattern\n");
    }
function static  error(str) char str[];
    {
    eprintf(str);
    }
