.bp
4.  Built in macros
.fo ''-#-'4.0'

     The   following   is   a  description  of  a  language  translator
     constructed using Mp. It is supplied with Mp as the default  macro
     library,  that  is,  the one built into the Mp task. The library's
     control structures were  loosely  copied  from  the  C  and  Basic
     languages.  For  reasons  of  efficiency,  some  macros  have been
     incorporated into the  macro  processor.  The  document  hards.doc
     describes  the  various  internal  routines  that  implement macro
     expansions. 

     In the following description:

	     repeat*            repeated 0 or more times
             repeat+            repeated 1 or more times
	     thing              a syntactic item (non-terminal)
             OTHER STUFF        literal string if upper case
	     {   }              optional 0 or 1 time
	     {   }*             grouping repeated 0 or more
	     {   }+             grouping repeated 1 or more


4.1     Language description
.fo ''-#-'4.1'

    The language is compatible with the standard vax calling sequence.
    It  always  generates  a  stack  call  (calls)  but  references to
    arguments are through the ap so the caller can be fortran or other
    languages. The  code  that  is  generated  is  generally  position
    independent,  but  use  of  absolute  symbol  references  must  be
    monitored  by  the  user  to  create  truly  position  independant
    routines. 

    In most cases the normal assembler modes can be  used  wherever  a
    variable  is permitted, but those with side affects (auto increment
    etc.) usually will not work. However, the expander does not  check
    for  this situation. The []'s are reserved for use by the language
    and thus indexed mode cannot be directly used. There  is  however,
    the escape mechanisms of ~ in column 1 and the i%dout statement. 

    The language requires the user to supply lexical information, that
    is,  all  tokens must be separated by spaces or commas.

    Symbols  used  inside  functions may be reused in other functions.
    However, undeclared variables in one function which are  declared
    in  another  function  will  cause  undetectable (at compile time)
    errors. Variable names may be declared to be  register  variables,
    which  will  map  to  registers  r11  down,  and the expander will
    allocate temporary registers from rx on up where the user can  set
    the first register (see the program statement). Any registers below
    rx  can  be  used  freely  by  the  user,  but  r0  and r1 must be
    referenced only in limited ways (see below). Functions always save
    all registers (r2 to r11) on entry. 

    If the optimizer is to be used, the user must not  use  r0  or  r1
    except  in  limited  ways  as  described  in  section 4.9. This is
    because the code generation uses r0 and r1 for  specific  purposes
    and  thus  enables  the  optimizer  to operate on a window of code
    without the need  to  be  very  smart.  Since  the  optimizer  can
    generally  be  expected  to  reduce  the total size of a module by
    10-20 percent, it is  well  worth  the  penalty  of  minor  coding
    restrictions. 



.ne 37
4.2     Program structure.
.fo ''-#-'4.2'



   |------------------------------------------------------------|
   |                                                            |
   |               Required program structure                   |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
   |       program Name            |used once                   |
   |                               |                            |
   |       .long ...               |------------                |
   |       .ascii ...              | static and/or              |
   |       .byte...                | global data                |
   |                               |------------                |
   |FUNCTION entry[p1,p2 ...]      |                            |
   |       auto l1 l2 l3 ...       | Function 1                 |
   |       register  x1 x2 x3 ...  |                            |
   |       external  y1 y2 y3 ...  |                            |
   |       {                       |                            |
   |       ...                     | code is here               |
   |       return  value           |                            |
   |       }                       |                            |
   |                               |-------------               |
   |FUNCTION entry2[p1,p2 ...]     |                            |
   |       auto l1 l2 l3 ...       |                            |
   |       register  x1 x2 x3 ...  | Function 2                 |
   |       {                       |                            |
   |       ...                     | code is here               |
   |       return   value          |                            |
   |       }                       |                            |
   |                               |-------------               |
   |                                                            |
   |------------------------------------------------------------|

4.3  Data types
.fo ''-#-'4.3'

     The  language  supports only longwords, words, and bytes. There is
     no concept of integer,  real,  etc.  Just  bit  streams.  Data  is
     usually  accessed by longword but may also be referenced as arrays
     and structures. Even if a variable is allocated as a  longword  it
     can  still be treated as an array of 4 bytes.   However, no bounds
     checking is done. 

          The data can, however, be in 4 modes. These are: 

	        Hardware register
	        Function parameter
	        Local stack allocated data
	        Static local or external data.

     The  translator  will  determine from a symbol table which mode is
     required of a variable and perform different transformations.  For
     example,  a  variable  in  a register must be mapped to the actual
     register name  rx  and a function parameter must be changed  to  a
     displacement  value  off  of  the  hardware ap register. Likewise,
     local data is a displacement to  the  frame  pointer  (fp).  These
     transformations  occur  automatically, even if vax machine opcodes
     are specified to operate on  these  mp  variables.  Some  special
     operations  can  be  performed  on  some  of  these types. This is
     described below.

     a.   The & operator

          This  permits  the  computation  of the address of a local or
          parameter variable  in  assignment  statements  and  function
          parameters. 

          For example, if b is a local 

			      a := &b

          This will generate code which puts the address of b into a. 

		       a := function.of[c,d,&e]

          This invokes `function.of' passing c, d, and the  address  of
          e. The function returns a value in r0 which is stored in a. 

     b.   The @ operator

          This permits the  indirection  thru  a  register,  local,  or
          parameter variable. (@a[b] etc. is not allowed).

          For example

			      a := @b

     c.   Array references

			array[index{,l/w/b}]

          where  the default is l (long word). For example a byte array
          reference would look like: 


			array[index,b]

          And a long word reference as: 

			array[index]

			    or

			array[index,l]

          If  array  is  a parameter variable, then the contents of the
          array   parameter are used as a pointer  to  an  array.  Auto
          variables are accessed directly. 

          There  is  an  implied  conversion  of signed byte to long or
          signed word to long,  so  that  mixed  mode  expressions  are
          permitted. 

          Since array notation is similar to a function call, any array
          variable used must be declared. The default interpretation is
          the function, but external functions should be declared. 

          Auto  variables  may  be  declared  as pointer variables with
          respect to how they are  treated   by   the  array  operator.
          Normally an auto array is declared with a specified value (in
          units  of  bytes, words or longwords ) which causes the stack
          to be decremented on function  entry  (by  the  corresponding
          amount).  Alternatively  one  can  declare  a single longword
          variable as *var which only  is  meaningful  with  the  array
          operator.  In  that  event,  the  auto variable is treated as
          though it were a single parameter variable which contains the
          address of an  array.  When  a  variable  declared  to  be  a
          register  is  used  in  an  array  reference,  the same thing
          happens, i.e. the register's contents are used as the address
          of the actual data. 

		Example:

		function test[x,y]
			Auto a,*b,c
                        {
			   b     := x + y
			   b[a]  := c
			}

          This  would  cause  the  contents  of  x  plus  y to be added
          producing a value which is the address of  the  beginning  of
          some  array.  Then, b[a] would reference the a'th longword in
          that array. Note that if b were declared a register then  the
          same  effect  would  result,  but  different  code  would  be
          generated - hopefully shorter. 

          The arguments of an array or function can be either a term or
          an expression in <>'s. It is invalid to enclose a single term
          (or  a  function  call or array reference) in <>. I.e. the <>
          pair is used to signify an expression. 

     d.   Member references

		       #member[pointer]

			      or

		      #member[pointer,l/w/b]

          This notation is an extended displacement mode, that  is,  it
          takes  the  member name as an offset (this may be a constant)
          and adds it to the pointer value (which may be an  expression
          etc.) and uses that address to access a long, word, or byte. 

	  Example:

		a := #left_link[pointer]
		x := #right_link[#left_link[node]]
		byte := #access_code[system_structure,b]

4.4    Constants
.fo ''-#-'4.4'

      a.  Strings

          A string is defined to be: 

	     "any string of ascii characters not including d-quote"

          The string is passed onto macro as: 

		       .asciz _ascii string_

          one  can  imbed non-printing characters (or " or _ itself) as
          follows: 

		       _<###>_ is a single character.


          Example:

			"This string contains a line-feed here _<12>_"


          Since all library routines expect strings  to  be  passed  by
          descriptor, the string notation of 

                       "__string text"

          will be converted to 

                       .ascid _string text_

	  There is limitted suport for string literals of the form:

		       'a' , 'aa', 'aaa',  or 'aaaa'

          These are mapped into #^A'...'. Two characters  are  actually
          allowed  the  ` and '. Whichever starts the string is used to
          generate  the  output  format.  The  last  character  is  not
          actually  examined  (though  it must be present) and could be
          anything. For example, the following are valid: 

		       `a', 'a', `a`

          The limitation is that the characters  permitted  inside  the
          quotes may not be space, tab, comma, or left square bracket. 

       b. Numbers

          Numerical constants have the following format: 

		a. #xxx                xxx = any assembler expression
		b. {-}{digit}+         like 52 or -8090 or 0
		c. 0{digit}+           like 0377 or 0177777


          The #xxx form is also used to get  the  address  of  external
          data  or  for  the value of global constants. If the #^A/.../
          form is used, then there may be no spaces, commas,  tabs,  or
          left square brackets. 


4.5  Declarative statements
.fo ''-#-'4.5'


o    PROGRAM {string} {rx}  {options}

          This statement must be the first statement in a program.
          It initializes the library and some labels  used  later.
          It  generates  a  .title  using string, uses string for
          label generation, and sets switch 15-21 to their initial
          values. If string is not entered then the  label  prefix
          must  be  setup  using SETLAB (see macros for more info)
          and no .title is output. 

          Rx is optional and specifies the first  register  to  use
          for temporaries. The default is r2.

          Options may be parameters 3 - n  and  currently  include
          the  value GLOBALS. This suppresses the .disable globals
          that is normally output to force definitions of external
          symbols. Since much of the dec macros require the global
          default  for  undefined  symbols,  this  is  a  hook  to
          suppress  this safety measure. One could also simply add
          an .enable global statement inline after this statement. 


o    FUNCTION name[parameter-label*]    {is local}

          This  is  the  first  statement  in  a  function. It can
          optionally declare that name  is  not  to  be  a  global
          variable. 

o    AUTO  label+


          This  statement declares local stack frame variables. The
          default size is 4 bytes. To allocate a different  amount
          use the following format. 

		  AUTO var[amount{,type}] ...

          The  type  may  be byte, word, or long (the default) and
          only the first character is significant. 

          Autos  may  also  be  declared  as pointers. This has an
          affect only when used in an array context. The method of
          declaring is as: 

		  Auto *var

o    REGISTER  var+

          Declares  variable  names to be equated to registers r11
          down. During output, the registers  actual  notation  is
          substituted. Only about 6 register variables are allowed
          and depends on the first temp register used. See Program
          above.  If  too many register variables are declared for
          the available registers, an error is generated  and  the
          user  must change some register variable declarations to
          Autos. 

o    EXTERNAL  var+

          This  declares  variables   or   procedures   that   are
          statically  defined elsewhere in the program (or another
          file). The format for functions is: 

			External func[*]

          Definition of all  symbols  and  external  functions  is
          suggested.  Note  that  a  conditional statement results
          which declares  the  parameters  external  if  they  are
          undefined. This assumes that variables declared external
          (to  functions) might be defined above in the same file.
          If the user  were  to  define  the  external  after  the
          declaration,  then  the  assembler  would  do  different
          actions on pass 1 and  2,  with  unpredictable  results.
          This  would  of  course  be  an  improper  use  of  this
          preprocessor, but would  probably  not  be  detected  at
          compile or assembly time. 

.ne 5
o    STRUCT name {initial_offset}
o    MEMBER name+
o    END-STRUCT {name}


          These  three  macros  provide  an  easy  way to generate
          offset values for some structure. It simply equates  the
          names  (labels)  to  absolute  values  (starting  at the
          initial offset which defaults to 0).  The  name  of  the
          structure  is just another label which is equated to the
          length of all the labels declared with the MEMBER macro. 

          Example:

		STRUCT person
		  MEMBER name[20,bytes],address[30,bytes],zip[10,bytes]
		  MEMBER state[1,w],sex[1,w]
		  MEMBER occupation_code,birthdate[6,bytes]
                END-STRUCT person

          The following label assignments  would  occur:  (all  in
          decimal) 

		  name=        0    (size =   20   bytes)
                  address=    20              30
                  zip=        50              10
                  state=      60               2
                  sex=        62               2
                  occupation= 64               4
                  birthdate=  68               6

		  and person would be 74 (length of all fields)

          These labels may be used as offsets to index  registers
          or  member references. The structure name may be used to
          allocate the necessary amount of storage  to  accomodate
          the structure. 

	  Example:

		 bill     := alloc[#person]     ;storage allocater.
		 #sex[bill,w] := `m'            ;set field
		 addr_zip := bill + #zip        ;address of bill's zip

	  Note that if bill were declared to be a register variable, and
	  the optimizer is used, then many statements reduce to a simple
	  displacement mode reference.


.ne 15
4.6   Executable statements
.fo ''-#-'4.6'


o    RETURN  {Expr}

          Returns  from  a function with and optionally computes a
          value for r0. 

	  For example, a complete (nonsense) program:


	     program sample

     function main[argc,argv]       ;function with 2 parameters
	 auto i,a,b,j,k             ;local variables
	 {
	 do while a ne b
	     a := argc - 1
             b := a - 1
	     loop for i := b downto 0
                 k := k + 1
		 loop for j := k  to  a    step two[k]
                    if strlen[argv[i]]  = strlen[argv[j]]
			 printf "(%s) = length of (%s)",argv[i],argv[j]
                    fi
                 next j
             next i
	 od
	 return 1
	}

     function two[j]                   ;double routine
	     {
	     return  j * 2
             }
             .end




.ne 30

o Assignment statement.


	 Assignment_statement ::= assignment  | function_call
	 assignment           ::= dest := right
	 right                ::= expr | term
	 expr                 ::= term {operator term}+
         term                 ::= source | function_call | array | member

	 source               ::= variable | constant | register | string
	 dest                 ::= variable | register | array | member

	 function_call        ::= function_name[arglist]
	 arglist              ::= argument {,argument}*
	 argument             ::= <expr> | term

	 array                ::= variable_name[argument{,L|W|B}]
	 member               ::= #element[argument{,L|W|B}]

         operator             ::= + | - | / | *


          Dest may not appear on the right  side  unless  its  the
          first  term  after  the  `:='. Only 6 terms are allowed.
          When referencing functions or  arrays,  r0  and  r1  are
          used. Expressions require the use of temporary registers
          which  are  allocated  automatically. Recursive function
          calls also require temporaries. Evaluation  is  strictly
          left  to  right.

	  Examples:

                 a := b - c / d
		 a := b - fun[a,1,&c] * fun2[loc(r5),par(r5)]
		 s := "s gets address of this asciz string"
		 r := val[argv[1],10] + -5
		 r := func[array[<c - f[d]>]]
		 r := #member[array[<c - f[d]>]]
		 a := b
		 s := &b + @a
		 f[x]

          Two  intrinsic  functions  exist. They are: Ref and Bits. Ref
          takes a single argument (expression or term) and produces  as
          a   function  return  (in  r0)  the  address  of  a  longword
	  containing the value of the expression or term.

		Example:

			  fortran_routine[ref[a],ref[1]...]


          Bits returns an extracted bit field as a  long  integer  into
          r0. Its format is as follows: 

                       bits[term,start_bit,number_bits]

          where  term is either a single variable or an expression, and
          start_bit and number_bits are simple variables, registers, or
          constants (no expressions or functions).  This  function   is
          also  a  special  case  function which can appear on the left
          side of an assignement statement. If term is a variable, then
          start_bit need not be 0-31 but can be an  index  into  a  bit
          array. 


	  For example:

			Auto a[4],b[4]
			   ...
			loop for i := 0 to 127
			    bits[a,i,1] := bits[b,i,1]
                        next i

          This  example  will  copy a 128 bit array from b into a. Note
          the allocation of 4 longwords for  each  variable,  and  that
          auto variables can be referenced in the bit function. 


o    IF(B) condition {OR|AND condition ...

             or

o    IF condition-code



    relation ::= NE | EQ | LT | HI | LO | LOS | HIS | GT | GE | LE | =
    condition-code ::= relation  | CC | CS  | ON  | OFF

    condition ::= value relation value
    value     ::= variable | constant | register | <expr> | array
			   | function | member

          (The IFB statement does not support <expr>, array, member, or
          function.) 

          This  sets  up  an  if-block. Condition is evaluated right to
          left with respect to OR/AND; Only four conditions may  appear
          on one line. 



.ne 17
          Example:

                        IF a = b   AND  c = d   OR   e = f

          is equivalent to a ()'ed boolean

                           IF (a=b AND (c=d   OR (e=f)))


          The on/off are used for bit testing.

          Example:

                                  bit #4,variable
                                  IF ON
                                  ...
                                  FI


          Expressions  are  the  same  as in assignment statements, but
          must be inclosed in <>'s. 

	  Example:

		IF <a + b> GT <c - d>   AND   array[<i + 1>] NE #0

		if a[0,b] =  `a'        ;these two are equivalent
		ifb a  =  `a'           ;but ifb is more efficient



o    ELSEIF(B) condition ...


          alternate if block, conditions as in an IF


o    ELSE

	  default selection within an IF block (must follow all
          elseif's)

o    FI

          terminate an if block


.ne 10
          Example:

		       IF this = r0    OR   foo NE 5
			  IF nested = example[function[i]]
                          ...
                          ELSEIF jim GT john
                          ...
			  ELSEIF jim LE john  AND i NE 1
                          ...
                          ELSE
                          ...
                          FI

                       ELSEIF autovar HIS parameter
                          ...
                       ELSE
                          ...
                       FI

.ne 10
o    SELECT CASE variable OF


          introduce a case construct


.ne 10
o    CASE(B) operand {OR}

          One or more case statements follow the select statement.
          Each selects a block of code conditionally by  comparing
          the   SELECT   variable   with  operand.  Multiple  case
          statements may be OR'd with a single block  of  code  by
          using  the  OR  keyword. The last case in the group does
          not have the  OR.  Note  that  a  BREAK  or  some  other
          transfer  of control must follow the block of code since
          `drop-thru' is undefined. 

          The  operand  is  anything  permitted in the destination
          field of a cmp(b) machine instruction. 


.ne 10
o    DEFAULT


          executes the default code for a case statement.   This
          must be the last selector in the SELECT group.


.ne 20
o    ESAC


          terminates a case construct.


.ne 16
          Example:


			 SELECT case (r4) of
            
                           CASEB  `a'
                              ...
                              BREAK
            
                           CASEB  `b' OR
                           CASEB  `c'
                              ...
                              BREAK
            
                           DEFAULT
                         ESAC

          The  select/case  group  is  a  more  compact  form of a
          multiway IF block. However,  the  individual  selections
          must  be  terminated  by an explicit transfer of control
          (usually via a BREAK). 

.ne 10

o    LOOP for indx = ini TO|DOWNTO|THRU|DOWNTHRU end {STEP amt}

	      indx ::= variable | register
	      ini  ::= <expr> | term
              amt  ::= <expr> | term
              end  ::= <expr> | term

          (THRU and DOWNTHRU generate unsigned branches)



          Negative going loops must supply  the  step  amount.  It
          defaults  to  1 (regardless of the existence of a DOWNTO
          or DOWNTHRU keyword). Loop is top tested; hence, a  loop
          may be iterated zero or more times. 

.ne 10

o    NEXT indx


          At  the  end of a loop body, this adds the step value to
	  the  indx  and returns to the top of the loop.  indx  is
          checked for proper nesting. 



.ne 20


          Example:


	     LOOP for i := #alpha  THRU  #beta  STEP  2    ;step by words

             ...
                 IF ...
                    BREAK                                  ;terminates loop
                 FI
             NEXT i







o    DO WHILE(B) if-condition


          Loop  as  long as condition is met. if-condition is an
          IF statement type condition. Loop is top tested. 


o    DO {FOREVER}

          sets up an infinite loop which is only ended via  a  jmp
          or BREAK. The keyword FOREVER is optional.


o    OD


          end of the do loop.


          Example:

             DO FOREVER                      DO WHILE a = b
               ...                              ...
               BREAK IF a NE b               OD
             OD


o    BREAK {IF(B) ...}


          This causes a controlled  jump  to  the  first  statement
          following  a  DO WHILE, DO FOREVER, LOOP, or SELECT CASE
          construct.  It leaves only the innermost  loop  or  case
          construct  currently  active.   If  an  IF  statement is
          supplied then it is equivalent to the form:

                  IF(B) ...
                     BREAK
                  FI


4.7  Vax opcodes and Mp variables
.fo ''-#-'4.7'

     The  statement  processor also checks to see if the statement is a
     one to four operand vax instruction. This  it  determines  by  the
     argument  count  and the fact that it is not a defined macro or an
     opcode which begins with `.' (dot). If it is a vax opcode then the
     same rules for variables and constants apply here as well. That is
     it is NOT just output untouched but is translated into actual  vax
     type operands. For example: 

.ne 8
	f:      .word 0                 ;the `.' protected the 0
					;from becoming #^o0
		function test[d,e]
		Auto a,b,c
		{

		movl 0377,e     ;generates movl #^O0377,e(ap)
		movl 25,a       ;generates movl #^D<25>,a(fp)
		addl3 25,a,b    ;generates addl3 #^D<25>,a(fp),b(fp)
		...

     This  means that if you define a macro for macro-32 to process and
     invoke it with operands like constants  and  variables  they  will
     also  be  modified  just  like the bit instruction. If this is not
     desired then the escape operation of i%dout must be used: 
.ne 3

		i%dout <mymacro 0>      ;generates mymacro 0
		~       mymacro 0       ;works also


     This  feature can also be turned off via the /na switch. Note that
     only simple variables are  transformed,  that  is,  no  functions,
     double  quoted  strings (single quoted are ok), or expressions are
     allowed. 

     Since  Vms  makes  heavy use of keyword macros for system services
     and Rms, a macro called ARGS  exists  which  will  check  for  the
     symbol  after the `=' and perform symbol transformations using the
     #@{x}  argument  expander  described  in  section  2.9.  Thus  for
     example: 

	...
	Auto nblks
	Register addr
	External FabBlock
	...

	$fab_store                                      -
	    args fab=FabBlock alq=nblks                 -
	    args fna=@addr    fns=20
	$create                                         -
            args fab=FabBlock
	...

     In  this  example,  nblks,  addr  and  20 will be transformed into
     macro-32 syntax. 

     Note  the  `-'  at  the  end  of  continued  lines. The one on the
     $fab_store and $create lines are necessary, while the ones on  the
     args lines are only needed to continue the line. These hyphens are
     handled  by  mp  macros and thus must not be in contact with a `;'
     unlike the situation in macro-32. 


4.8  Notes
.fo ''-#-'4.8'

     Some  stack  and  switch  dump macros exist as well: Dump, dmpstr,
     dmpsw, peek and poke. These are  usually  used  to  test  out  new
     macros  interactively. Setlab allows one to set the string used to
     make labels. 

     The macros make use of the two stacks  internally.  Stack  one  is
     used  for  all "IF" constructs and all others use stack 2. This is
     so that a BREAK can be nested inside an  IF  block  and  find  the
     innermost LOOP or CASE branch-to label.

.ne 35

     Stack usage is as follows:




              If (stack 1)
                     tos     branch to on false
                     nos     go here on done (after elseif's etc)

              case (stack 2)
                     tos     break to label
                     nos     variable to test
                     3os     skip over to next case test label
                     4os     Case true (the Or label)

              Do while/forever (stack 2)
                     tos     break to
                     nos     jmp back to test or continue in loop label

              loop (stack 2)
                     tos     break to
                     nos     jmp back to test for more looping label
                     3os     loop index
                     4os     step parameter


4.9  optimizer
.fo ''-#-'4.9'

    The  code generated by the macros is in some cases rather poor. In
    the area of expressions and array/member references many temporary
    register  uses  can  be eliminated. Also, branching is designed to
    never have to worry about reaching code. Rather than slow down the
    macro generation or generate all short branches, it was felt  that
    an  optional optimizer pass could be called upon after a module is
    debugged. Thus good code generation and fast  translation  can  be
    both attained. 

    However, a peephole optimizer can be very difficult to write if it
    needs to rigorously analyse register usage. Instead, the optimizer
    for  MP  knows  the manner of usage of indivdual registers and can
    thus delete unecessary code or reduce the complexity  of  operands
    where a registers contents are known in advance. 

    For  example,  r0  and r1 are used in array calculations by always
    producing the subscript in r1 and then using index  mode  to  move
    the  value  of  the  array element into r0. The code generator can
    then simply view an array reference in the same way as a  function
    call. However, this usually generates code as follows: 

		movl subscript,r1
		movl array[r1],r0
		movl r0,destination

    This can be easily folded into the more efficient sequence: 

		movl subscript,r1
		movl array[r1],destination

    And  if  the subscript is the constant zero, then the macros would
    have generated the following: 

		clrl r1
		movl array[r1],r0
		movl r0,destination

    This is optimised in two passes into the following: 

	first
		movl array,r0           ;since [r1] is just 0
		movl r0,destination
	second
		movl array,destination  ;by above folding of r0

    Thus  a  personalized  peephole  optimizer  need  only  look   for
    sequences of MP code to produce rather good results. 

    However, if the user were to naivly generate code that expected r0
    to  stay  set,  then incorrect code could result. Thus in order to
    use the optimizer, the user should never manipulate r0  except  as
    follows: 

	First move r0 or r1 to another register as follows:

		moval (r1),r2           ;same as movl r1,r2
		moval (r0),r2           ;same as movl r0,r2

	To set r0 or r1 use the following:

		addl3 r2,#0,r0
		addl3 r2,#0,r1


	Also, the bits intrinsic function will work:


		bits[r0,0,32] := x

		     or

		x := bits[r1,0,32]

    But  notice  that the reference to r1 above will clobber r0. Also,
    one can create macro-32 macros or opcodes that are unknown to  the
    optimizer which can reference these registers. 

    The  best  way  to  be sure is to check the modules opt*.mac which
    each describe the particular optimizations performed. 

    The branch optimizer is currently very simpleminded. It creates  a
    symbol  table using the information on the PROGRAM line to be able
    to spot internally  generated  labels.  Then  2  passes  are  made
    looking for word displacement branches that can be changed to byte
    displacement.  It guesses that byte branches can reach 50 lines in
    either direction. If an assembler error occurs use the -l#  option
    with a value less than 50 (or more if you dare). 

    The optimizer reads the  entire  file  (source.mpp)  into  virtual
    memory as a doubly linked list and makes about 14 passes (order is
    important)  modifing  the  code.  Finally it outputs the optimized
    version (source.opt). The command file op.com and the command:


		@op source  [-option] [-option]  ...

    is used to run the program, and then run macro-32. The following
    options are supported:

		-v      verbose
		-l#     set the number of lines assumed to be
			reachable with a short branch to #.
		-o#     do only pass # (for debugging mostly)


.bp

.fo ''-#-'5.0'
5.   Language Summary


           PROGRAM string
	   FUNCTION parameter-label*
           AUTO  label+
           REGISTER  label+
           EXTERNAL  label+
	   RETURN  {Expression}

           STRUCT name
           MEMBER name+
           END-STRUCT {name}

	   destination := term  {operator term}*

	   IF|IFB condition {OR|AND condition ...
           IF condition-code
           ELSEIF(B) condition ...
           ELSE
           FI

	   SELECT CASE variable OF
           CASE(B) operand {OR}
           DEFAULT
           ESAC

	   LOOP FOR indx = ini TO|DOWNTO|THRU|DOWNTHRU end {STEP amt}
	   NEXT indx

	   DO WHILE(B) if-condition
           DO FOREVER
           OD

	   BREAK {IF(B) ...}

           PUSH|POP  variable
.bp
.fo ''-#-'6.0'
6.  Alternate built in macro libraries


     The  default  library  is  built in at task build time. The module
     mcmd.mac contains a macro called "mc". This  module  is  assembled
     with  all text for a macro library as calls to the "mc" macro (one
     per line of macros in the library). The  "mc"  macro  simply  puts
     some  pointers  around  the  text  which  is  assembled  as .ascii
     statements. Then at run time, these are loaded from an overlay and
     scanned (as if they came from a file). The main  reason  for  this
     method  of  loading  is speed. Note that the /NI switch suppresses
     this loading. Useful when testing out a new  library.  The  module
     mcmd.mac is assembled as: 

			  >mcr mac  mcmd=mcmd.

     Then the MP task is rebuilt: 

			  >mcr tkb @mp.bld.

     The mcmd.obj module is NOT put in a library. 

     One other source module is available which has  some  labels  that
     allocate  more  or  less  storage for various mp internals and the
     area for storage of macro text. Use the /un switch to measure  the
     remaining  storage  available  after  a library is loaded. If many
     levels  of  recursive  macro  expansion  are  required,  bump  the
     stack=xxx  in  the  build file. The amount of storage used is kept
     track of and the lowest sp value detected is output at the end  of
     each  run.  Running out of space may cause program failures, so be
     generous. The module stak.mac also contains storage which  can  be
     increased. It is assembled thus: 

                         >mcr mac stak=ml/ml,stak.

     It  also is not put into a library. If the processor blows up when
     trying to initialize itself (loading default macros) use  the  /is
     switch  to  have  the  lines echoed on the tty so one may see what
     line in the mcmd.mac file caused the problem. If the program  runs
     out  of space for macro library text it usually just crashes. Bump
     the amount of text (in bytes) and the number of lines allowed  for
     macros. 

     Note that some macros have been coded directly into the source for
     increased  speed.  Even  with  a  totally redefined default set of
     macros, one would still have a few built in macros. If  one  needs
     to  get  rid  of these too, then the best way is to find the table
     macro call (which enters  the  name  into  the  opcode  table)  in
     stak.mac  and  change  the macro from table to NOtable. Since some
     routines in the let*.mac group have found uses in  other  modules,
     it  is best to leave the source code alone, even though most of it
     would then no longer be used. Special care must be taken to  place
     any  new  modules  on the main overlay path with all the recursive
     routines (the path with stat* and let*). New opcodes can be  added
     by  finding  a  slot  in stak.mac and then placing an entry in the
     corresponding  dispatch  table  in  stat.mac.  The  slots  can  be
     extended  at  the end also simply by adding an entry at the end of
     each table (the assembler will compute the number of entrys in the
     each). 

.ne 30
     The  task  build  file for Mp also contains global definitions for
     the various characters used to delimit parameters. If  one  wishes
     to  have  >  and >= etc. as operators instead of GT or GE then the
     enclosing brackets would need to be changed from <>'s to something
     else, say for example {}'s. Any macro library used would of course
     have to  conform  to  these  characters  as  well.  The  character
     definitions for the unary operators also are found there. 

     Note  that  any  change in the delimiter characters would probably
     result in some Macro-32  incompatibility.


.fo ''-#-'6.1'
6.1 Modifications to internals

     Most sources  for vax mp are written in a prior version of mp that
     generates pdp11 code. The pdp11 version is not compatible with the
     vax version, although its close. Many poor design  decisions  were
     corrected  in  the  vax  version.  In  order to rebuild any of the
     sources for vax mp,  one  must  use  the  image  rmp.exe  and  the
     following commands: 

		rmp file/lm/nb/nl
		mcr mac file=file.mpp

     this will translate file.mac into file.obj. 
.ne 16

     All global storage for stacks, opcodes,  and  numerical  data  are
     found in stak.mac. 

     The following lists some of the modules that may be of interest to
     a user who wishes to modify vax mp. 

	stat            statement expander - recursive decent parser
        stat1           opcode lookup (hash tables etc)
        stat2           string stack operators
        stat3           assignment := parsing

        let,let1-3      various portions of the assignment statement

        sub             parameter expansion and substitution

        hards           secondary dispatcher for hardcoded macros

        macexp          macro expansion module
        nums            internal numerical variables and s%n exprs

     The rmp.exe image is built from the spring 1981 rsx decus tape.

.bp
.fo ''-#-'7.0'
7.0 Examples

     The  following  is  an  example  of  a  Macro  created to supply a
     construct similar to the C and Ratfor FOR loop. Recall the  syntax
     of the FOR is: 

	For (initialize ; test ; reinitialize)
           statement

     The  easiest  way  to  implement  this statement, is to modify the
     syntax to keep within the allowable constructs  of  MP.  Thus  our
     version will be: 

     

	FOR <initialize> <WHILE(B) test> <reinitialize>

	ROF

     Any of the three parts of a FOR loop  may  be  null  and  will  be
     interpreted as in C and RATFOR. 

     Note  the requirement that the keyword WHILE(B) be explicit in the
     test part of the macro call. This  is  to  avoid  needing  a  FORB
     macro.  The  following  macro definitions supply such a statement,
     using the already existing macros in the default  macro  library.

     The  initialize  part  is  expanded on a line by itself. The i%din
     statement is used since the expansion can be any statement at  all
     (even an assignment statement). 

     The test part is converted into a DO WHILE(B) loop.  If  the  test
     part  is  null,  then  it  is  converted  into  FOREVER  which  is
     consistent with the definition of the FOR loop. This  is  done  by
     placing  either  parameter  2  onto  the  stack or if it was null,
     FOREVER is pushed there. Then the stack is poped   into  parameter
     2 replacing any previous contents. 

     The reinitialize part of the call is pushed in its  entirety  onto
     the  string  stack.  The  ROF  macro simply pops the statement and
     expands it. All the leg work for label generation  is  handled  by
     the DO and OD macros. 

.ne 10

		   $MACRO FOR <init-1> <condition-2> <reinit-3>
		   i%din <#@1>
		   I%B  <#@2> <?- FOREVER>  <?- <#@2>>
		   ?+ 2
                   DO  #@2
                   ?- <#@3>
                   $ENDM
                    
                   
                   
                   $MACRO ROF
                   ?+ 1
		   i%din <#@1>
                   OD
                   $ENDM

     A valid macro call using this construct could be: 


		    FOR <i := 0> <WHILE i LT 10> <incl i>
                    ...
                    ROF
.bp

     The  following  program  demonstrates  using  some  of  the system
     library routines under Vms and is presented as a sample program. 


	  program print r5                      ;local use of r4 and r3
          
	  function print[string,arg1]           ;print formatted string
          auto out[520,bytes],len
          register bbb
          {
               bbb := desc[&out,520]                 ;build a descriptor
	       r4 := sys$faol[string,&out,bbb,&arg1] ;address of arg1 is
	       r3 := lib$put_output[bbb]             ;address of the
	  }                                          ;actual parameter block
          
          
          
          function desc[of,len]                      ;builds descriptors
          {                                          ;for strings
	      of[0] := len
              of[2,b] := 14
              of[3,b] := 2
              of[1] := of + 8
              return of
          }


          function syserr[str,number]                ;looks up a system
          auto buf[200,bytes],len,stat2              ;error code and outputs
          {                                          ;text with an optional
              desc[&buf,200]                         ;user string
              sys$getmsg[number,&len,&buf,1,&stat2]  ;trans error to ascii
              buf[0,w] := len                        ;set length
              if str = 0
                 print["__!AS",&buf]                 ;just print error
              else
                 print["__!AS !AS",str,&buf]         ;print err and user
              fi                                     ;string
          }
          .end

.bp

    The  following  code  is  generated from the above. Note the label
    generation uses label=. rather than label: so local symbol  blocks
    are  not affected.  This output is after the optimizer pass.

;	 Mpvax version: 23-July-81 translation: 08-AUG-81 16:13:01
		.ident		/08-AUG-81_16:13:01/
		
		.disable		global
		.enable		debug,suppression
		.title		print
		
		_temp_		= -10
		_top_		= .
		_auto_		= _temp_
		
		
; Function print[string,arg1]
		
		_auto_		= _temp_
		.long           2
		
		string		=	4
		arg1		=	8
		
		.entry		print,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		_auto_		= _auto_-<<520>*1>
		out		=	_auto_
		
		_auto_		= _auto_-4
		len		=	_auto_
		
		
		addl		#_auto_,sp
		
		pushl		#^D<520>
		pushal		out(fp)
		calls		#2,desc
		movl		r0,r11
		pushal		arg1(ap)
		pushl		r11
		pushal		out(fp)
		pushl		string(ap)
		calls		#4,sys$faol
		movl		r0,r4
		pushl		r11
		calls		#1,lib$put_output
		movl		r0,r3
		ret
		
		
		
		
; Function desc[of,len]
		
		_auto_		= _temp_
		.long           2
		
		of		=	4
		len		=	8
		
		.entry		desc,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		
		addl		#_auto_,sp
		
		moval		@of(ap),r5	
		movl		len(ap),(r5)
		movl		#^D<2>,r1
		movab		@of(ap)[r1],r5	
		movb		#^D<14>,(r5)
		movl		#^D<3>,r1
		movab		@of(ap)[r1],r5	
		movb		#^D<2>,(r5)
		movl		#^D<1>,r1
		moval		@of(ap)[r1],r5	
		movl		of(ap),r6
		addl		#^D<8>,r6
		movl		r6,(r5)
		movl		of(ap),r0
		ret
		
; Function syserr[str,number]
		
		_auto_		= _temp_
		.long           2
		
		str		=	4
		number		=	8
		
		.entry		syserr,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		_auto_		= _auto_-<<200>*1>
		buf		=	_auto_
		
		_auto_		= _auto_-4
		len		=	_auto_
		
		_auto_		= _auto_-4
		stat2		=	_auto_
		
		
		addl		#_auto_,sp
		
		pushl		#^D<200>
		pushal		buf(fp)
		calls		#2,desc
		pushal		stat2(fp)
		pushl		#^D<1>
		pushal		buf(fp)
		pushal		len(fp)
		pushl		number(ap)
		calls		#5,sys$getmsg
		movaw		buf(fp),r5	
		movw		len(fp),(r5)
		cmpl		str(ap),#^O0
		Bneq		print1		
print4=.
		.save
		.psect		textstrings
print5=.
		.ASCID		_!AS_
		.byte		0
		.restore
		pushal		buf(fp)
		pushal		print5
		calls		#2,print
		brb		print6		
print1=.
		.save
		.psect		textstrings
print7=.
		.ASCID		_!AS !AS_
		.byte		0
		.restore
		pushal		buf(fp)
		pushl		str(ap)
		pushal		print7
		calls		#3,print
print6=.
print3=.
		ret
;%%reg%% 5 11
		.end		
.bp

    program tape r5
    
    
    function tape
    auto buf[1000,b],chan
    {
        chan :=    mtassign[ref[0]]          ;assign channel quietly
	print["__ size = !SL",taperead[ref[chan],&buf,ref[1000],ref[0]]]
    }
    
    
    function mtassign[verbose]                 ;call by ref for fortran use
    auto chan,s
    {
	s := sys$assign["__MTA0",&chan,0,0]
    	if @verbose = 1
                syserr["__assign to mt returns=",s]
    	fi
        return chan
    }
    
    function taperead[chan,buf,size,verbose]    ;also call by ref
    auto ios[4,w],s                             ;so use @chan etc.
    {
    
	s := sys$qiow[1,@chan,#io$_readlblk,&ios,0,0,buf,@size,0,0,0,0]
        if @verbose = 1                 ;print info if asked
	    syserr["__qio returns=",s]
            print["__ios is`!UW !UW'",ios[0,w],ios[1,w]]
            print["__buf is`!OW !OW'",buf[0,w],buf[1,w]]
            syserr["__ios block of qio=",ios]
        fi
	if ios[0,w] ne #ss$_normal
	    return -1                   ;return -1 on eof or error
	else
            return ios[1,w]             ;else the record size if ok
	fi
    }
    .end  tape
.bp
;	 Mpvax version: 23-July-81 translation: 08-AUG-81 16:04:39
		.ident		/08-AUG-81_16:04:39/
		
		.disable        global
		.enable		debug,suppression
		.title		tape
		
		_temp_          = -10           ;start of local vars
		_top_		= .
		_auto_          = _temp_        ;init local counter
		
; Function tape[]
		
		_auto_          = _temp_        ;init local storage counter
		.long           0               ;number of arguments
		
		.entry		tape,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		_auto_		= _auto_-<<1000>*1>
		buf		=	_auto_
		
		_auto_		= _auto_-4
		chan		=	_auto_
		
		addl            #_auto_,sp      ;allocate local storage
		
		.save
		.psect		temps
tape1:          .long 0
		.restore
		clrl		tape1
		pushal		tape1		
		calls		#1,mtassign
		movl		r0,chan(fp)
		.save
		.psect		textstrings
tape2=.
		.ASCID		_ size = !SL_
		.byte		0 ;null terminate also
		.restore
		.save
		.psect		temps
tape3:	 .long 0
		.restore
		clrl		tape3
		pushal		tape3		
		.save
		.psect		temps
tape4:	 .long 0
		.restore
		movl		#^D<1000>,tape4
		pushal		tape4		
		pushal		buf(fp)
		.save
		.psect		temps
tape5:	 .long 0
		.restore
		movl		chan(fp),tape5
		pushal		tape5		
		calls		#4,taperead
		pushl		r0	
		pushal		tape2
		calls		#2,print
		ret
		
; Function mtassign[verbose]
		
		_auto_		= _temp_ ;init local storage counter
		.long		1			;number of arguments
		
		verbose		=	4
		
		.entry		mtassign,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		_auto_		= _auto_-4
		chan		=	_auto_
		
		_auto_		= _auto_-4
		s		=	_auto_
		
		addl		#_auto_,sp ;allocate local storage
		
		.save
		.psect		textstrings
tape6=.
		.ASCID		_MTA0_
		.byte		0 ;null terminate also
		.restore
		pushl		#^O0
		pushl		#^O0
		pushal		chan(fp)
		pushal		tape6
		calls		#4,sys$assign
		movl		r0,s(fp)
		cmpl		@verbose(ap),#^D<1>
		Bneq		tape7		
tape10=.
		.save
		.psect		textstrings
tape11=.
		.ASCID		_assign to mt returns=_
		.byte		0 ;null terminate also
		.restore
		pushl		s(fp)
		pushal		tape11
		calls		#2,syserr
tape7=.
tape9=.
		movl		chan(fp),r0
		ret
		
; Function taperead[chan,buf,size,verbose]
		
		_auto_		= _temp_ ;init local storage counter
		.long		4			;number of arguments
		
		chan		=	4
		buf		=	8
		size		=	12
		verbose		=	16
		
		.entry		taperead,^m<r2,r3,r4,r5,r6,r7,r8,r9,r10,r11>
		_auto_		= _auto_-<<4>*2>
		ios		=	_auto_
		
		_auto_		= _auto_-4
		s		=	_auto_
		
		addl		#_auto_,sp ;allocate local storage
		
		pushl		#^O0
		pushl		#^O0
		pushl		#^O0
		pushl		#^O0
		pushl		@size(ap)
		pushl		buf(ap)
		pushl		#^O0
		pushl		#^O0
		pushal		ios(fp)
		pushl		#io$_readlblk
		pushl		@chan(ap)
		pushl		#^D<1>
		calls		#12,sys$qiow
		movl		r0,s(fp)
		cmpl		@verbose(ap),#^D<1>
		Bneq		tape12		
tape15=.
		.save
		.psect		textstrings
tape16=.
		.ASCID		_qio returns=_
		.byte		0 ;null terminate also
		.restore
		pushl		s(fp)
		pushal		tape16
		calls		#2,syserr
		.save
		.psect		textstrings
tape17=.
		.ASCID		_ios is`!UW !UW'_
		.byte		0 ;null terminate also
		.restore
		movl		#^D<1>,r1
		cvtwl		ios(fp)[r1],-(sp)		
		cvtwl		ios(fp),-(sp)		
		pushal		tape17
		calls		#3,print
		.save
		.psect		textstrings
tape18=.
		.ASCID		_buf is`!OW !OW'_
		.byte		0 ;null terminate also
		.restore
		movl		#^D<1>,r1
		cvtwl		@buf(ap)[r1],-(sp)		
		cvtwl		@buf(ap),-(sp)		
		pushal		tape18
		calls		#3,print
		.save
		.psect		textstrings
tape19=.
		.ASCID		_ios block of qio=_
		.byte		0 ;null terminate also
		.restore
		pushl		ios(fp)
		pushal		tape19
		calls		#2,syserr
tape12=.
tape14=.
		cvtwl		ios(fp),r5	
		cmpl		r5,#ss$_normal
		Beql		tape20		
tape23=.
		movl		#^D<-1>,r0
		ret
		brb		tape24		
tape20=.
		movl		#^D<1>,r1
		cvtwl		ios(fp)[r1],r5	
		movl		r5,r0
		ret
tape24=.
tape22=.
		ret
;%%reg%% 5 11                                   ;used by optimizer
		.end		tape
.bp
                                    6.  alternate built in macro libraries
                             2.9 macro  argument operators
             2.7 macro definitions and  arguments
                          6. alternate  built in macro libraries
                                    4.  built in macros
                     2.4 variables and  conditionals
                                   4.4  constants
                                   4.3  data types
                                   4.5  declarative statements
                             2.7 macro  definitions and arguments
                          4.1 language  description
                            2. what mp  does
                                    7.  examples
                                   4.6  executable statements
                          3. operating  instructions
                           2.5 more on  internal statements
                  6.1 modifications to  internals
                                    1.  introduction
                                   4.1  language description
           6. alternate built in macro  libraries
                                   2.9  macro argument operators
                                   2.7  macro definitions and arguments
                 6. alternate built in  macro libraries
                                    5.  macro summary
                               2.2 the  macros
                  2.3 treatment of non  macros
                           4. built in  macros
                                   6.1  modifications to internals
                                   2.5  more on internal statements
                               2. what  mp does
                   4.7 vax opcodes and  mp variables
                      2.3 treatment of  non macros
                                   4.8  notes
                               4.7 vax  opcodes and mp variables
                                    3.  operating instructions
                    2.9 macro argument  operators
                               2.1 the  pass
                                   4.2  program structure.
                        2.8 the string  stack
                  2.5 more on internal  statements
                       4.5 declarative  statements
                        4.6 executable  statements
                               2.8 the  string stack
                           4.2 program  structure.
                              5. macro  summary
                                   2.6  switches
                                   2.3  treatment of non macros
                              4.3 data  types
                                   2.4  variables and conditionals
                4.7 vax opcodes and mp  variables
                                   4.7  vax opcodes and mp variables
                                    2.  what mp does
