	PROGRAM PLOTLABEL
C
C   *SEGMENT (MAIN DRIVER FOR PLOT LABEL PACKAGE)
C
C	This program is designed to generate labels for Printronix and
C	Versatec plot outputs, which include the username, plot file name
C	and the date.
C
C	Author:
C	=======
C		Richard A. Olds
C		Air Force Weapons Laboratory
C		Laser Development Division
C		AFWL/ARLB
C		Kirtland AFB, NM 87117
C		(505)844-4691
C
	INCLUDE 'PLOTLABEL.CMM'
C
C   *** SET DATA FOR LABEL
	CALL SET_DATA
C
C   *** OPEN PROPER PLOT DEVICE
	CALL OPEN_DEV
C
C   *** NOW, PLOT LABELS ON PROPER DEVICE
	CALL PLOTTER
C
C   *** CLOSE UP PLOT FILE
	CALL DONEPL
C
	STOP
	END

	SUBROUTINE OPEN_DEV
C
C   *SEGMENT (OPEN PROPER PLOT DEVICE FOR LABELS)
C
	INCLUDE 'PLOTLABEL.CMM'
C
	IF (DEVICE .EQ. 'PNTRNXPOP') GOTO 10
	IF (DEVICE .EQ. 'VERSTCPOP') GOTO 20
	GOTO 30
C
C   *** PRINTRONIX 300/600 ***
10	CALL PRINTRONIX
	GOTO 40
C
C   *** VERSATEC 1200A ***
20	CALL VERSATEC
	GOTO 40
C
C   *** UNKNOWN DEVICE TYPE ***
30	PRINT 100, DEVICE
	STOP 'PLOTLABEL -- Warning, Unknown device type.'
C
40	RETURN
C
100	FORMAT (1H0,' Plot Label Error -- Unknown device type =',A,'=')
	END

	SUBROUTINE PLOTTER
C
C   *** SEGMENT (PLOT LABELS)
C
	INCLUDE 'PLOTLABEL.CMM'
C
	CHARACTER*8 CHR_TIME
	CHARACTER*9 CHR_DATE
C
	INTEGER IBUFF(20)
C
C   *** FIND OUT DATE AND TIME ***
	CALL DATE (CHR_DATE)
	CALL TIME (CHR_TIME)
C
C   *** PICK PROPER SCALING AND ORIENTATION FOR PLOT LABELS
C
	IF (DEVICE .EQ. 'PNTRNXPOP') GOTO 10
	IF (DEVICE .EQ. 'VERSTCPOP') GOTO 20
	GOTO 9000
C
C	Define plotter limits
C
C   *** PRINTRONIX 300/600
10	YPAGE = 10.5
	ANG = 90.0
	HT = 0.9
	GOTO 30
C
C   *** VERSATEC 1200A
20	YPAGE = 10.5
	ANG = 90.0
	HT = 0.9
C
30	HT2 = HT*0.75
	HT3 = HT*0.50
	XPAGE = 2.0*HT + 2.0*HT2 + 2.0*HT3 + 1.0
C
C   *** BEGIN PLOT PACKAGE ***
	CALL BGNPL (-1)
C
	XAREA = XPAGE - 0.5
	YAREA = YPAGE - 0.5
C
	CALL NOBRDR
	CALL GRACE (0.0)
	CALL PAGE (XPAGE, YPAGE)
	CALL AREA2D (XAREA, YAREA)
C
	CALL SWISSB
	CALL ANGLE (ANG)
	CALL HEIGHT (HT)
C
	L1 = 0
	DO 40 I=1, 12
		IF (USER_NAME(I:I) .EQ. ' ') GOTO 50
		L1 = I
40		CONTINUE
C
50	ENCODE (80, 1000, IBUFF) USER_NAME(1:L1)
	XPOS = 2.0 * HT
	RLEN = XMESS (IBUFF, 100)
	YPOS = YAREA*0.5 - RLEN*0.5
	CALL MESSAG (IBUFF, 100, XPOS, YPOS)
C
	L2 = 0
	DO 60 I=79, 1, -1
		L2 = I
		IF (FILE_ID(I:I) .NE. ' ') GOTO 70
60		CONTINUE
70	ENCODE (80, 1010, IBUFF) FILE_ID(1:L2)
D	PRINT *, L2, FILE_ID(1:L2)
C
	XPOS = XPOS + 2.0*HT2
75	CALL HEIGHT (HT2)
	RLEN = XMESS (IBUFF, L2)
D	PRINT *, RLEN, YAREA, HT2
	IF (RLEN .LE. YAREA) GOTO 80		! If too large, rescale, retry
	HT2 = 0.95*HT2
	GOTO 75
C
80	YPOS = YAREA*0.5 - RLEN*0.5
	CALL MESSAG (IBUFF, L2, XPOS, YPOS)
C
	CALL HEIGHT (HT3)
	ENCODE (80, 1020, IBUFF) CHR_DATE, CHR_TIME
	XPOS = XPOS + 2.0*HT3
	RLEN = XMESS (IBUFF, 100)
	YPOS = YAREA*0.5 - RLEN*0.5
	CALL MESSAG (IBUFF, 100, XPOS, YPOS)
C
	CALL ENDPL (0)
C
	RETURN
C
9000	STOP 'PLOTTER==FATAL, Improper Device Type for plot label.'
C
1000	FORMAT (A, '$')
1010	FORMAT (A)
1020	FORMAT (A, ' == ' A, '$')
	END

	SUBROUTINE SET_DATA
C
C   *** SEGMENT (READ DATA SPECIFYING LABEL ID AND DEVICE)
C
	INCLUDE 'PLOTLABEL.CMM'
C
C   *** DETERMINE USER'S NAME ***
	CALL GETJPI_USERNAME (0,USER_NAME)
D	PRINT *, USER_NAME
C
C   *** DETERMINE PLOT FILE NAME ***
	CALL TRANS_LOG_PHYS ('FILE_ID', FILE_ID)
D	PRINT *, FILE_ID
C
C   *** DETERMINE H/W DEVICE PLOT IS GOING TO ***
	CALL TRANS_LOG_PHYS ('DEVICE_ID', DEVICE)
D	PRINT *, DEVICE
C
	RETURN
	END
