C**************************************************************************
C
C	This procedure is explained fully in the huffinfo.txt file.
C	In short, it will huffman code all values in an input array
C	or will do a combination of huffman and huffman/fixed length 
C	coding. The routine is Fortran callable and must be supplied
C	with the following inputs: 1)size:size of input array buf,
C	2)bound:number of entries to be strictly huffman coded, 3)
C	inbuf:real array of inputs, 4)clen:integer array will hold 
C	the length of each code word upon exit from routine, 5)valx:
C	integer array will hold the code word for each entry upon exit  
C	from routine, 6)length:the length in bits of the fixed length  
C	code.
C	Douglas A. Ford         August 27, 1981 
C	Oak Industries Advanced Technology Group
C	Rancho,Bernardo,CA
C
C************************************************************************ 
	subroutine huff(size,bound,inbuf,clen,valx,length)
	integer size,bound,ent,locx(0:500,0:500),valx(0:size),p,
	1clen(0:size),change(0:500)
	real buf(0:500),inbuf(0:size),save(0:500),etr,sbuf(0:500)
	do k=0,size			!Total the entries so you can later
		ent=ent+inbuf(k)	! divide by it to get probabilities.
	end do
	do k=0,size			!Initialize locx array to all -1.
		do j=0,size
			locx(k,j)=-1
		end do
	end do
	do k=0,size			!Set up initial locations.
		locx(k,0)=k
	end do
	do k=0,size			
		buf(k)=inbuf(k)/ent	!Store probabilities in buf. 
		save(k)=buf(k)		!Save array used to compute entropy.
	end do
	p=0				!Set flag.
	s=size				!Set upper bound of loop.
198	low=0				!Set lower bound of loop.
200	t=-1				!Initial testor out of loop.
	do j=low,s-1			!Bubble sort.
		if(buf(j).gt.buf(j+1)) then	!If value should move up.
			temp=buf(j+1)		!Switch values.
			buf(j+1)=buf(j)
			buf(j)=temp
			t=j
			l=0			!Switch all locations.
			do while((locx(j+1,l).ne.-1).or.(locx(j,l).ne.-1))
				temp=locx(j+1,l)
				locx(j+1,l)=locx(j,l)
				locx(j,l)=temp
				l=l+1
			end do
		end if
	end do
	if(t.eq.-1)goto 127		!If no switches we're done jump out.
	s=t				!Reset upper bound of loop.
	goto 200			!Jump to beginning of loop.
127	if(p.eq.0) then			!Check flag, only done once.
		etr=0
		do k=bound,size
			etr=etr+buf(k)	!Sum up remaining probabilities. 
		end do
		sbuf(bound)=etr		!Enter sum as last probability.
		do k=bound+1,size	!Enter constituent locations.
			locx(bound,k-bound)=locx(k,0)
		end do
		do k=0,size		!Save constituent locations.
			change(k)=locx(bound,k)
		end do
		do k=0,bound-1		!Enter probabilities to be huff coded.
			sbuf(k)=buf(k)
		end do
		p=1			!Set flag.
		goto 198
	end if
	do k=0,bound-1			!Huffman code loop.
		sbuf(k+1)=sbuf(k)+sbuf(k+1)	!Add lowest probabilities.
		i=0
		do while(locx(k,i).ne.-1)	!Low prob(all locs) get a '1'.
			p=1
			p=ishft(p,clen(locx(k,i)))
			valx(locx(k,i))=ior(valx(locx(k,i)),p)
			clen(locx(k,i))=clen(locx(k,i))+1
			i=i+1
		end do	
		i=0
		do while(locx(k+1,i).ne.-1)	!High prob(all locs) get a '0'.
			clen(locx(k+1,i))=clen(locx(k+1,i))+1
			i=i+1
		end do
		do m=0,bound			!Transfer locs of low to high.
			if(locx(k,m).eq.-1)goto 45
			locx(k+1,i)=locx(k,m)
			i=i+1
		end do
45		do j=k+1,bound-1		!Bubble sort new list.
			if(sbuf(j).gt.sbuf(j+1)) then	!If vals should move up
			temp=sbuf(j+1)			!Switch values.
			sbuf(j+1)=sbuf(j)
			sbuf(j)=temp
			l=0				!Switch locations.
			do while((locx(j+1,l).ne.-1).or.(locx(j,l).ne.-1))
				temp=locx(j+1,l)
				locx(j+1,l)=locx(j,l)
				locx(j,l)=temp
				l=l+1
			end do
			else
				goto 134		!Shortened bub sort 
			end if				!already partially
		end do					!ordered.
134	end do
	if(bound.ne.size) then		!Assign proper vals to huff/fixed codes
		p=ishft(valx(change(0)),length)	!Shift huff code left.
		etr=clen(change(0))+length	!New length for each huff/fixed
		do k=0,size-bound
			valx(change(k))=ior(p,change(k))	!Assign new val
			clen(change(k))=etr		!Assign new length.
		end do
	end if
	end

