	program plot_q_manager
c
c	v 2.00
c	18-Nov-1981
c
c	author:	Gerson H Cohen
c		National Institutes of Health
c		Bldg 2, Room 312
c		Bethesda, MD  20205
c
c	purpose:To manage a queue of requests for despooling
c		plots on the Versatec printer/plotter,
c		handling other related functions such as:
c		ABORT, CANCEL, STATUS
c
c	v1.01 was DCL based, deriving from sugestions from
c	Versatec's published command file (v1.00).
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	external perm_mbx_ast,term_mbx_ast
	parameter prv$v_oper = '12'x
	parameter prv$v_pswapm = 'c'x
	parameter prv$v_sysprv = '1c'x
	parameter prv$v_tmpmbx = 'f'x
	parameter dib$k_length = '24'x, dib$w_unit = 'c'x
	integer*2 dib_buffer(dib$k_length/2)
	integer*4 dib_descr(2)
	data dib_descr(1)/dib$k_length/
	character*23 date_time
c
c	first let's get the name of this process correct
c
	call iferr ( sys$setprn ( 'plotqman' ) )
c
c	write a time stamp
c
	call lib$date_time ( date_time )
	write ( 6,10 ) date_time
10	format (' PLOTQMAN started: ',a)
c
c	we need a mailbox to receive the word from the user
c
	call iferr ( sys$crembx ( %val(1), perm_mbx_chan,
	1		%val(msg_size), %val(msg_size),
	1		%val(0), , 'PLOTTER_MAILBOX' ) )
c
c	assignment of the channel is implicit in the mailbox
c	creation. Now mark the mailbox for deletion so that
c	it may disapear if the process gets deleted.
c
	call iferr ( sys$delmbx ( %val(perm_mbx_chan) ) )
c
c	we need a mailbox to return information to the user
c
	call iferr ( sys$crembx ( %val(1), info_mbx_chan,
	1		%val(msg_size), %val(msg_size),
	1		%val(0), , 'PLOTTER_STATUS' ) )
c
c	assignment of the channel is implicit in the mailbox
c	creation. Now mark the mailbox for deletion so that
c	it may disapear if the process gets deleted.
c
	call iferr ( sys$delmbx ( %val(info_mbx_chan) ) )
c
c	we need a mailbox to return information to the user
c
	call iferr ( sys$crembx ( %val(1), term_mbx_chan,
	1		%val(acc$k_termlen), %val(acc$k_termlen),
	1		%val(0), , ) )
c
c	assignment of the channel is implicit in the mailbox
c	creation. Now mark the mailbox for deletion so that
c	it may disapear if the process gets deleted.
c
	call iferr ( sys$delmbx ( %val(term_mbx_chan) ) )
c
c	we need the unit of this mailbox
c	obtain via the $getchn mechanism
c
	dib_descr(2) = %loc(dib_buffer)
	call iferr ( sys$getchn ( %val(term_mbx_chan), ,
	1		dib_descr, , ) )
	term_mbx_unit = dib_buffer(dib$w_unit/2+1)
c
c	initialize our queue pointers and establish the
c	ast addresses
c
	plotter_running = .false.
	currently_processed = 0
	last_in = 0
	perm_mbx_ast_addr = %loc(perm_mbx_ast)
	term_mbx_ast_addr = %loc(term_mbx_ast)
c
c	set up the privelege mask for subprocess creation
c
c	give it OPER, PSWAPM, SYSPRV and TMPMBX privileges
c
	priv_mask(1) = ishft ( 1, prv$v_oper )
	1 		+ ishft ( 1, prv$v_pswapm )
	1 		+ ishft ( 1, prv$v_sysprv )
	1		+ ishft ( 1, prv$v_tmpmbx )
	priv_mask(2) = 0
c
c	now queue a mailbox read to the permanent mailbox
c	using the AST mechanism
c
	call set_perm_mbx
c
c	set an alarm clock to write a time stamp in the log
c
	call set_timer
c
c	now we're all primed and can just sleep until
c	something happens
c
	do while (.true.)
c
c		be sure ASTs are permitted
c
		call iferr ( sys$setast ( %val(1) ) )
c
c
c		let's try not to go to sleep if more work is pending
c
		if ( plotter_running .or.
	1		currently_processed.eq.last_in ) then
c
c			first purge the working set
c
			call purgws
c
			call iferr ( sys$hiber ( ) )
		end if
c
c		if we get here, let's see if something
c		can be done
c
c		check the plotter flag and the queue
c		pointers before committing ourselves
c
		if ( .not.plotter_running .and.
	1		currently_processed.ne.last_in ) then
c
c			AST recognition must be disabled to
c			prevent multiple calls to next_plot
c			before it gets a chance to set the
c			proper flags
c
			call iferr ( sys$setast ( %val(0) ) )
c
			call next_plot
		end if
	end do
	stop
	end
	subroutine purgws
c
c	subroutine to purge working set while just sitting around doing nothing
c
	integer*4 purge(2)
	data purge / 0, '7fffffff'x /
c
	call iferr ( sys$purgws ( %ref(purge) ) )
	return
	end
	subroutine set_timer
c
c	subroutine to set an alarm clock for the purpose of
c	writing a time stamp
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	external timer_ast
	character*14 interval
	data interval/'0 01:00:00.00'/
	integer time(2)
c
c	cancel any outstanding timer flags
c
	call iferr ( sys$cantim ( , ) )
c
c	convert delta time to system format
c
	call iferr ( sys$bintim ( interval, time ) )
c
c	set the clock and an AST
c
	call iferr ( sys$setimr ( , time, timer_ast, ) )
c
c	and purge the working set
c
	call purgws
	return
	end
	subroutine timer_ast
c
c	subroutine to respond to the timer AST and write the time stamp
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	character*23 date_time
c
	if ( plotter_running ) then
		continue	! we don't need this one in the log
	else
		call lib$date_time ( date_time )
		write ( 6,10 ) date_time
10		format ( ' Time stamp: ', a )
	end if
c
	call set_timer
	return
	end
	subroutine perm_mbx_ast
c
c	subroutine to get a message which has been send to
c	the plot_q_manager's permanent mailbox
c	and to decode it, possibly acting on it
c
c	TYPE will be used in this subroutine in place of
c	WRITE to 6 to avoid recursive i/o which would
c	occur when this subroutine is called during
c	execution of next_plot
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	character*23 date_time
	logical inp_ok
c
c	if we're here we have a message.  check that it
c	isn't blank and then strip any leading blanks
c
	if ( message.eq.' ' ) then
		call set_perm_mbx
		call purgws
		return
	end if
c
	call strip_blanks ( message )
c
c	convert the message to all caps
c
	call str$upcase ( message, message )
	call str$trim ( message, message, message_l )
	type 10, ' Received: ', message(:message_l)
10	format ( 2a )
c
c	read the clock
c
	call lib$date_time ( date_time )
	type 20, date_time
20	format ( ' Time: ', a )
c
c	PLOT - enter a name into the queue for despooling
c
	if ( message(1:5).eq.'PLOT ' ) then
		message = message(5:)
c
c		information must still remain in the message
c
		if ( message.eq.' ' ) then
			continue
		else
			call strip_blanks ( message )
			last_in = mod(last_in,num_msg) + 1
			the_queue(last_in) = message
c
c			wake up to get some work done
c
			call iferr ( sys$wake ( , ) )
		end if
	end if
c
c	ABORT - kill the current plot using a control-y
c
	if ( message(1:6).eq.'ABORT ' .and. plotter_running ) then
		message = message(7:)
		if ( message.eq.' ' ) then
			continue
		else
			call strip_blanks ( message )
			msg_end=index(message,' ')-1
			inp_ok = .false.
			read ( message(:msg_end), 30, err=40 ) the_pid
			inp_ok = .true.
30			format (z)
40			continue
			if ( inp_ok ) then
c
c				accomplish using force_exit with control-y
c				resulting in %SYSTEM-W-CONTROLY code
c
				if ( pid.ne.0 .and. pid.eq.the_pid ) then
					call iferr ( sys$forcex ( the_pid, ,
	1						%val('610'x) ) )
				end if
			end if
		end if
	end if
c
c	STATUS - tell what is in the queue
c
	if ( message(1:7).eq.'STATUS ' ) then
		open ( unit=8, name='PLOTTER_STATUS', status='old' )
		i=currently_processed
		if ( plotter_running .or. i.ne.last_in ) then
			do while ( i-1.ne.last_in )
				qs=index(the_queue(i),' ')-1
				if ( plotter_running .and.
	1				i.eq.currently_processed ) then
					type 50, 'Current', i,
	1					the_queue(i)(:qs), pid
					write ( 8,50 ) 'Current', i,
	1					the_queue(i)(:qs), pid
				else
					type 50, 'Pending', i,
	1					the_queue(i)(:qs)
					write ( 8,50 ) 'Pending', i,
	1					the_queue(i)(:qs)
				end if
				i=mod(i,num_msg)+1
			end do
		else
			type 50, 'Nothing in queue'
			write ( 8,50 ) 'Nothing in queue'
50			format ( x, a, i4, 2x, a, :, 5x, 'PID =', z9.8  )
		end if
		close ( unit=8 )
	end if
c
c	CANCEL - blank a name already in the_queue
c
	if ( message(1:7).eq.'CANCEL ' ) then
		message = message(8:)
		if ( message.eq.' ' ) then
			continue
		else
			call strip_blanks ( message )
			msg_end=index(message,' ')-1
			inp_ok = .false.
			read ( message(:msg_end), 60, err=70 ) blank_it
60			format (i)
			inp_ok = .true.
70			continue
			if ( inp_ok .and. blank_it.gt.0 .and.
	1			blank_it.le.num_msg ) then
				the_queue(blank_it) = ' '
			end if
		end if
	end if
c
c	reset mailbox and return if command not found
c
	call set_perm_mbx
	call set_timer
	return
c
	end
	subroutine strip_blanks ( string )
c
c	subroutine to strip leading blanks from a string
c
	character*(*) string
c
	i = len(string)
	do while ( i.gt.0 .and. string(1:1).eq.' ' )
		string = string(2:)
		i = i - 1
	end do
	return
	end
	subroutine set_perm_mbx
c
c	subroutine to issue a read with AST trap to the
c	permanent mailbox
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
c
	message=' '		! erase previous message
	call iferr ( sys$qio ( , %val(perm_mbx_chan),
	1	io$_readvblk, , %val(perm_mbx_ast_addr), ,
	1	%ref(message), %val(len(message)), , , , ) )
c
	return
	end
	subroutine next_plot
c
c	subroutine to do the actual despooling of a plot
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	character*(msg_size) source, working, version, log_file
	logical exists
	character*23 date_time
c
	plotter_running = .true.
	currently_processed = mod(currently_processed,num_msg) + 1
c
c	now ASTs are possible
c
	call iferr ( sys$setast ( %val(1) ) )
	working = the_queue(currently_processed)
	if ( working.eq.' ' ) then	! this must have been cancelled
		plotter_running = .false.
		return
	end if
	write ( 6,30 ) ' Working on: ', working(:index(working,' ')-1)
c
c	get the SOURCE: and version number
c
	source_l = index(working,']')
	if ( source_l.gt.0 ) then
		source = working(:source_l)
	else
		source = ' '
	end if
	semi = index(working,';')
	if ( semi.gt.0 ) then
		version=working(semi:index(working,' ')-1)
	else
		version = ' '
	end if
	version_l=index(version,' ')-1
c
c	let us open the plot files before continuing to test
c	that they are still around
c
	exists = .false.
	inquire ( file=source(:source_l)//'PARM.PLV'//
	1	version(:version_l), name=working, exist=exists,
	1	err=10 )
	if ( exists ) write ( 6,30 ) ' Found: ',
	1	working(:index(working,' ')-1)
10	continue
c
	if ( exists ) then
		exists = .false.
		inquire ( file=source(:source_l)//'VECTR1.PLV'//
	1		version(:version_l), name=working,
	1		exist=exists, err=20 )
		if ( exists ) write ( 6,30 ) ' Found: ',
	1		working(:index(working,' ')-1)
20		continue
	end if
	if ( .not.exists ) then
		write ( 6,30 ) ' The spooled files defined by ',
	1	source(:source_l)//'*.PLV'//version(:version_l),
	1	' are not found'
30		format ( 3a )
		plotter_running = .false.
		return			! exit sheepishly
	else
c
c		having a real name now,
c		reassess SOURCE and VERSION
c
		source_l = index(working,']')
		if ( source_l.gt.0 ) then
			source = working(:source_l)
		else
			source = ' '
		end if
		semi = index(working,';')
		if ( semi.gt.0 ) then
			version=working(semi:index(working,' ')-1)
		else
			version = ' '
		end if
		version_l=index(version,' ')-1
		write ( 6,30 ) ' Source: ',source(:source_l)
		write ( 6,30 ) ' Version: ',version(:version_l)
	end if
c
c	now write the command file to get the job done
c
	open ( unit=1, name='db:[plot]doplot.com', status='new',
	1		carriagecontrol='list' )
	inquire ( unit=1, name=working )	! get the full name
	work_name_l = index(working,' ') - 1
c
c	make a name for the log file
c
	log_file = working(:index(working,']'))//'doplot.log'
	log_file_l = index(log_file,' ')
c
	write ( 1,40 ) source(:source_l), version(:version_l),
	1		working(:work_name_l)
40	format ('$! set noverify'/'$ set noon'/
	1	'$ set process/noswap'/
	1	'$ show process/all'/
	1	'$ source :== ',a/
	1	'$ version:== ',a/
	1	'$ com_fil:== ',a/
	1	'$ @sys$manager:plotit'/
	1	'$ delete ''source''*.PLV''version'''/
	1	'$ delete ''com_fil''')
	close (unit=1)
c
c	create the subprocess
c
	call iferr ( sys$creprc ( pid, 'sys$system:loginout',
	1	working(:work_name_l), log_file(:log_file_l),
	1	log_file(:log_file_l), priv_mask, %val(0), 'doplot', 
	1	%val(6), , %val(term_mbx_unit), %val('0'x) ) )
c
c	log this
c
	write (6,50) pid
50	format (' PID',z9.8,' started')
c
c	get the time
c
	call lib$date_time ( date_time )
	write ( 6,60 ) date_time
60	format ( ' Time: ', a )
c
c	issue a read with AST on the termination mailbox
c
	call iferr ( sys$qio ( , %val(term_mbx_chan),
	1	io$_readvblk, , %val(term_mbx_ast_addr), ,
	1	%ref(term_msg), %val(acc$k_termlen), , , , ) )
c
c	now we can go back to sleep
c
	return
	end
	subroutine term_mbx_ast
c
c	subroutine to answer the AST from a termination mailbox
c
	implicit integer*4 (a-z)
	common / queue / the_queue, message, term_msg
	common / queue_pointers / currently_processed, last_in,
	1			plotter_running, pid
	common / ast_addresses / perm_mbx_ast_addr,
	1			term_mbx_ast_addr
	common / channels / perm_mbx_chan, term_mbx_chan, term_mbx_unit
	common / priv / priv_mask(2)
	parameter msg_size = 100, num_msg = 100
	character*(msg_size) the_queue(num_msg)
	character*(msg_size) message
	parameter acc$k_termlen = '54'x
	character*(acc$k_termlen) term_msg
	integer*2 perm_mbx_chan, term_mbx_chan, term_mbx_unit
	external io$_readvblk
	character*250 error_msg
	parameter acc$l_finalsts = '4'x
	parameter acc$l_pid = '8'x
c
c	zero the PID so we know the job is out
c
	pid = 0
c
c	copy the desired information out of the message string
c
	read ( term_msg(acc$l_finalsts+1:),10 ) final_status
	read ( term_msg(acc$l_pid+1:),10 ) zpid
10	format ( a )
c
c	log the completion
c
	write ( 6,20 ) final_status, zpid
20	format (' The status code', z9.8, ' was returned for PID',
	1 			z9.8 )
	if ( .not.final_status ) then
		call sys$getmsg ( %val(final_status),
	1		error_msg_l, error_msg, %val(15), )
		write (6,30) error_msg(:error_msg_l)
30		format (' The status is:'/a)
	end if
c
c	note that the plotter is idle
c
	plotter_running=.false.
c
c	wake up so that we can do more plotting if necessary
c
	call iferr ( sys$wake ( , ) )
c
	return
	end
